package mi.hdm.recipes;

import mi.hdm.exceptions.InvalidCategoryException;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

public class CategoryTest {
    Categories underTest = Categories.getInstance();

    @Test
    public void testEquals() {
        underTest.clearCategories();
        Category c1 = new Category("Abend", 0x00FF00);
        Category c2 = new Category("LEcker", 0x13);
        Category c3 = new Category("ew", 0x13);
        Category c4 = new Category("ew", 0x100);

        assertNotEquals(c1, c2);
        assertEquals(c3, c4);
        assertEquals(c2, c3);
    }

    @Test
    public void shouldNotBeAbleToAddIdenticalName() {
        underTest.clearCategories();
        underTest.addCategory("Mittag", 0x100000);
        underTest.addCategory("Mittag", 0x100040);

        Assert.assertEquals(1, underTest.getAllCategories().size());
    }

    @Test
    public void shouldNotBeAbleToAddIdenticalCode() {
        underTest.clearCategories();
        underTest.addCategory("Morgen", 0x100040);
        underTest.addCategory("Mittag", 0x100040);

        Assert.assertEquals(1, underTest.getAllCategories().size());
    }

    @Test
    public void shouldBeAbleToAddCategory() {
        underTest.clearCategories();
        underTest.addCategory("Morgen", 0x100000);
        underTest.addCategory("Abend", 0x100010);

        Assert.assertEquals(2, underTest.getAllCategories().size());
    }

    @Test
    public void canRemoveCategoryByName() {
        //given
        underTest.clearCategories();
        underTest.addCategory("Jan ist cool", 0xFF0000);

        //when
        underTest.deleteCategory("Jan ist cool");

        //expect
        assertEquals(0, underTest.getAllCategories().size());
    }

    @Test(expected = InvalidCategoryException.class)
    public void canNotAddCategoryWithEmptyName() {
        Category emptyName = new Category("", 0x1000);
    }

    @Test(expected = InvalidCategoryException.class)
    public void canNotAddCategoryWithNullName() {
        Category emptyName = new Category(null, 0x1000);
    }

    @Test(expected = InvalidCategoryException.class)
    public void canNotAddCategoryWithNegativeCode() {
        Category emptyName = new Category("foo", -100);
    }

    @Test(expected = InvalidCategoryException.class)
    public void canNotAddCategoryWithCodeTooHigh() {
        Category emptyName = new Category("foo", Integer.MAX_VALUE);
    }
}
